/**************************************************************************
*
* Filename:     CAClass.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Klasse zur Ermittlung und zur Darstellung der Benutzer des Systems
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace ManagedMSI
{
	using System;
	using System.Collections;
	using System.Collections.Specialized;

	/// <summary>
	/// Klasse zur Ermittlung und zur Darstellung der Benutzer des Systems
	/// </summary>
	public class CAClass
	{
		#region Konstanten
		/// <summary>
		/// Fehlernummer, falls weitere daten vorliegen
		/// </summary>
		private const int ERROR_NO_MORE_ITEMS = 259;

		/// <summary>
		/// Keine Fehler
		/// </summary>
		private const uint ERROR_SUCCESS = 0;

		/// <summary>
		/// Fehler
		/// </summary>
		private const int ERROR_INSTALL_FAILURE = 1603;

		/// <summary>
		/// Meldung fr ungltigen Modus
		/// </summary>
		private const string INAVLID_MODE_TEXT = "Die Funktion kann in diesem Modus nicht ausgefhrt werden. Es ist erforderlich sie whrend der Skriptausfhrung (Deferred Execution) zu verwenden.";

		#endregion

		#region Methoden
		/// <summary>
		/// Enumeriert die registrierten Benutzer des Systems und gibt diese in einer
		/// Liste aus
		/// </summary>
		/// <param name="hInstall">Handle der Installer Session</param>
		[DllExportAttribute()]
		public static uint ListAccounts(uint hSession)
		{
			// Konstanten
			const int MSIMODIFY_INSERT_TEMPORARY = 7;

			// Deklarationen
			IntPtr hView;
			int counter = 0;

			// Datenbank bestimmen
			IntPtr hDatabase = NativeMethods.MsiGetActiveDatabase(new IntPtr(hSession));

			// View ffnen
			if (NativeMethods.MsiDatabaseOpenView(hDatabase, "SELECT * FROM ListBox", out hView) == ERROR_SUCCESS)
			{
				// Alle Benutzerkonten des Systems abrufen
				foreach(UserAccount account in ManageAccounts.EnumerateAccounts())
				{
					// Ermittelte Daten der Tabelle temporr anfgen
					IntPtr hRecord = NativeMethods.MsiCreateRecord(4);
					NativeMethods.MsiRecordSetString(hRecord, 1, "SelectedUser");		// Property
					NativeMethods.MsiRecordSetInteger(hRecord, 2, counter);				// Ordner
					NativeMethods.MsiRecordSetString(hRecord, 3, account.Name);			// Value
					NativeMethods.MsiRecordSetString(hRecord, 4, account.Name);			// Text
					NativeMethods.MsiViewModify(hView, MSIMODIFY_INSERT_TEMPORARY, hRecord);
					MsiFunctions.Close(hRecord);

					// Counter aktualisieren
					counter+=1;
				}
				// Handle des Views schlieen
				MsiFunctions.Close(hView);

			}
			// Rckgabe
			return (ERROR_SUCCESS);
		}


		/// <summary>
		/// Legt ein neus Benutzerkonto an
		/// </summary>
		/// <param name="hSession">Handle der Installer Session</param>
		[DllExportAttribute()]
		public static uint CreateAccount(uint hSession)
		{
			// Prfen auf verzgerte Ausfhrung
			if (!MsiFunctions.GetRunMode(hSession, MsiFunctions.InstallRunMode.Scheduled))
			{
				// Meldung ins Protokoll schreiben
				MsiFunctions.WriteLogEntry(hSession, INAVLID_MODE_TEXT, MsiFunctions.InstallMessage.Error );  
				return (ERROR_INSTALL_FAILURE);
			}

			// Eigenschaft CustomActionData abrufen, parsen und Argumente in Dictionary bertragen
			StringDictionary parameters = MsiFunctions.GetCustomActionData(hSession);

			try
			{
				// Account hinzufgen
				ManageAccounts.CreateAccount(parameters["Name"], parameters["Description"], parameters["Password"]);
			}
			catch (Exception ex)
			{
				MsiFunctions.WriteLogEntry(hSession, ex.Message, MsiFunctions.InstallMessage.Error);
				return (ERROR_INSTALL_FAILURE);
			}

			// Rckgabe
			return (ERROR_SUCCESS);
		}

		/// <summary>
		/// Entfernt ein Benutzerkonto
		/// </summary>
		/// <param name="hSession">Handle der Installer Session</param>
		[DllExportAttribute()]
		public static uint RemoveAccount(uint hSession)
		{
			// Prfen auf verzgerte Ausfhrung
			if (!MsiFunctions.GetRunMode(hSession, MsiFunctions.InstallRunMode.Scheduled))
			{
				// Meldung ins Protokoll schreiben
				MsiFunctions.WriteLogEntry(hSession, INAVLID_MODE_TEXT, MsiFunctions.InstallMessage.Error );  
				return (ERROR_INSTALL_FAILURE);
			}

			// Eigenschaft CustomActionData abrufen, parsen und Argumente in Dictionary bertragen
			StringDictionary parameters = MsiFunctions.GetCustomActionData(hSession);

			try
			{
				// Account entfernen
				ManageAccounts.RemoveAccount(parameters["Name"]); 
			}
			catch (Exception ex)
			{
				// Fehler ins Protokoll schreiben
				MsiFunctions.WriteLogEntry(hSession, ex.Message, MsiFunctions.InstallMessage.Warning );  
			}
			return (ERROR_SUCCESS);
		}
		#endregion
	}
}
