/**************************************************************************
*
* Filename:     CANative.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Funktionen zum Zugriff auf den Windows Installer
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace ManagedMSI
{
	using System;
	using System.Text;
	using System.Collections.Specialized;
	using System.Runtime.InteropServices;

	#region Windows Installer Session
	/// <summary>
	/// Funktionen zum Zugriff auf den Windows Installer
	/// </summary>
	internal class MsiFunctions
	{
		#region Enumerationen
		/// <summary>
		/// Type der Meldung die verwendet werden soll
		/// </summary>
		internal enum InstallMessage : int
		{
			FatalExit      = 0x00000000,
			Error          = 0x01000000,
			Warning        = 0x02000000,
			User           = 0x03000000,
			Info           = 0x04000000,
			FilesInUse     = 0x05000000,
			ResolveSource  = 0x06000000,
			OutOfDiskSpace = 0x07000000,
			ActionStart    = 0x08000000,
			ActionData     = 0x09000000,
			Progress       = 0x0A000000,
			CommonData     = 0x0B000000,
			Initialize     = 0x0C000000,
			Terminate      = 0x0D000000,
			ShowDialog     = 0x0E000000,
		}

		/// <summary>
		/// Stellt den Ausfhrungsmodus dar
		/// </summary>
		internal enum InstallRunMode : int
		{
			Admin           =  0,
			Advertise       =  1,
			Maintenance     =  2,
			RollbackEnabled =  3,
			LogEnabled      =  4,
			Operations      =  5,
			RebootAtEnd     =  6,
			RebootNow       =  7,
			Cabinet         =  8,
			SourceShortNames=  9,
			TargetShortNames= 10,
			Windows9x       = 12,
			ZawEnabled      = 13,
			Scheduled       = 16,
			Rollback        = 17,
			Commit          = 18,
		}

		#endregion

		/// <summary>
		/// Property lesen
		/// </summary>
		/// <param name="Session">Handle auf die Installer Session</param>
		/// <param name="Name">Name</param>
		/// <returns>Wert</returns>
		internal static string GetProperty(uint hSession, string Name)
		{	
			IntPtr hInstall = new IntPtr(hSession);

			StringBuilder buf = new StringBuilder();
			uint bufSize = 0;
			uint ret = NativeMethods.MsiGetProperty(hInstall, Name, buf, ref bufSize);
			if(ret == (uint) NativeMethods.Error.MORE_DATA)
			{
				buf.Capacity = (int) ++bufSize;
				ret = NativeMethods.MsiGetProperty(hInstall, Name, buf, ref bufSize);
			}
			if(ret != 0)
			{
				throw new InstallerException(ret); 
			}
			return buf.ToString();
		}

		/// <summary>
		/// Property setzen
		/// </summary>
		/// <param name="Session">Handle auf die Installer Session</param>
		/// <param name="Name">Name</param>
		/// <param name="Value">Wert</param>
		internal static void SetProperty(uint hSession, string Name, string Value)
		{
			IntPtr hInstall = new IntPtr(hSession);

			uint ret = NativeMethods.MsiSetProperty(hInstall, Name, Value);
			if(ret != 0)
			{
				throw new InstallerException(ret); 
			}
		}

		/// <summary>
		/// CustomActionData abrufen und in einzelne Parameter umwandeln
		/// </summary>
		/// <param name="Session">Handle auf die Installer Session</param>
		/// <returns>StringDictionary</returns>
		internal static StringDictionary GetCustomActionData(uint hSession)
		{	
			// CustomActionData abrufen
			string customActionData = MsiFunctions.GetProperty(hSession, "CustomActionData"); 

			// CustomActionData parsen und in StringDictionary berfhren 
			return (Parser.ParseCommandLine(customActionData)); 
		}


		/// <summary>
		/// Ermittelt den Ausfhrungsmodus
		/// </summary>
		/// <param name="Session">Handle auf die Installer Session</param>
		/// <param name="modeFlag">Modus der geprft werden soll</param>
		/// <returns>True oder False</returns>
		internal static bool GetRunMode(uint hSession, MsiFunctions.InstallRunMode modeFlag)
		{
			return NativeMethods.MsiGetMode( new IntPtr(hSession), (uint) modeFlag);
		}

		/// <summary>
		/// Schreibt spezifische Meldungen ins Installationsprotokoll
		/// </summary>
		/// <param name="Session">Handle auf die Installer Session</param>
		/// <param name="messageTest">Meldung</param>
		/// <param name="errorMessage">Meldungstyp</param>
		internal static void WriteLogEntry(uint hSession, string messageTest, MsiFunctions.InstallMessage messageType)
		{
			IntPtr hInstall = new IntPtr(hSession);

			// Ermittelte Daten der Tabelle temporr anfgen
			IntPtr hRecord = NativeMethods.MsiCreateRecord(0);

			// Text in Record bertragen
			NativeMethods.MsiRecordSetString(hRecord, 0, messageTest);

			int ret = NativeMethods.MsiProcessMessage(hInstall, (uint)messageType, hRecord);
			MsiFunctions.Close(hRecord);
			
			if(ret < 0)
				throw new InstallerException((uint)ret); 
		}

		/// <summary>
		/// Schliet das Handle des Installer Objektes
		/// </summary>
		/// <param name="handle">Handle</param>
		internal static void Close(IntPtr handle)
		{
			if(handle != IntPtr.Zero)
			{
				uint ret = NativeMethods.MsiCloseHandle(handle);
				handle = IntPtr.Zero;
			}
		}
	}
	#endregion

	#region Native
	/// <summary>
	/// Native Funktionen
	/// </summary>
	internal sealed class NativeMethods
	{
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiGetProperty(IntPtr hInstall, string szName, StringBuilder szValueBuf, ref uint cchValueBuf);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiSetProperty(IntPtr hInstall, string szName, string szValue);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiOpenDatabase(string szDatabasePath, string szPersist, out IntPtr hDatabase);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiOpenDatabase(string szDatabasePath, IntPtr uiOpenMode, out IntPtr hDatabase);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiDatabaseOpenView(IntPtr hDatabase, string szQuery, out IntPtr hView);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiViewExecute(IntPtr hView, IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiViewFetch(IntPtr hView, out IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiViewModify(IntPtr hView, int iModifyMode, IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern int  MsiViewGetError(IntPtr hView, StringBuilder szColumnNameBuffer, ref uint cchBuf);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiViewGetColumnInfo(IntPtr hView, uint eColumnInfo, out IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern IntPtr MsiCreateRecord(uint cParams);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiFormatRecord(IntPtr hInstall, IntPtr hRecord, StringBuilder szResultBuf, ref uint cchResultBuf);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordClearData(IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordGetFieldCount(IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern bool MsiRecordIsNull(IntPtr hRecord, uint iField);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern int  MsiRecordGetInteger(IntPtr hRecord, uint iField);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordGetString(IntPtr hRecord, uint iField, StringBuilder szValueBuf, ref uint cchValueBuf);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordSetInteger(IntPtr hRecord, uint iField, int iValue);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordSetString(IntPtr hRecord, uint iField, string szValue);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordDataSize(IntPtr hRecord, uint iField);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordReadStream(IntPtr hRecord, uint iField, byte[] szDataBuf, ref uint cbDataBuf);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiRecordSetStream(IntPtr hRecord, uint iField, string szFilePath);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiCloseHandle(IntPtr hAny);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiCloseAllHandles();
		[DllImport("kernel32.dll", SetLastError=true)] internal static extern uint   FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern IntPtr MsiGetLastErrorRecord();
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern uint MsiDatabaseCommit(IntPtr hDatabase);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern IntPtr MsiGetActiveDatabase(IntPtr hInstall);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern int  MsiProcessMessage(IntPtr hInstall, uint eMessageType, IntPtr hRecord);
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] internal static extern bool MsiGetMode(IntPtr hInstall, uint iRunMode);


		/// <summary>
		/// Errors
		/// </summary>
		internal enum Error : uint
		{
			SUCCESS                =    0,
			MORE_DATA              =  234,

		}
	}
	#endregion

	#region Exceptions

	/// <summary>
	/// Klasse zur Behandlung von nativen Exceptions
	/// </summary>
	[Serializable]
	public class InstallerException : SystemException
	{
		/// <summary>
		/// Message formatieren
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		internal static extern uint FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);

		/// <summary>
		/// Instanz der Exception erstellen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		public InstallerException(uint errorCode):base(GetSystemMessage(errorCode)){}

		/// <summary>
		/// Systemfehlermeldung erzeugen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		/// <returns>Meldung</returns>
		private static string GetSystemMessage(uint errorCode)
		{
			const uint FORMAT_MESSAGE_IGNORE_INSERTS  = 0x00000200;
			const uint FORMAT_MESSAGE_FROM_SYSTEM     = 0x00001000;
			StringBuilder buf = new StringBuilder(1024);
			if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				IntPtr.Zero, (uint) errorCode, 0, buf, (uint) buf.Capacity, IntPtr.Zero) != 0)
			{
				return buf.ToString().Trim();
			}
			else
			{
				return null;
			}
		}
	}

	#endregion
}
