/**************************************************************************
*
* Filename:     NativeMethods.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Enthlt native Funktionen fr den Windows Installer
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace PatchModell
{
	using System;
	using System.Text;
	using System.Runtime.InteropServices;

	#region Installermethoden
	internal class MsiFunctions
	{
		/// <summary>
		/// Property auslesen
		/// </summary>
		/// <param name="Session">Session-Handle</param>
		/// <param name="Name">Name</param>
		/// <returns>Wert</returns>
		internal static string GetProperty(uint Session, string Name)
		{	
			IntPtr hInstall = new IntPtr(Session);

			StringBuilder buf = new StringBuilder();
			uint bufSize = 0;
			uint ret = NativeMethods.MsiGetProperty(hInstall, Name, buf, ref bufSize);
			if(ret == (uint) NativeMethods.Error.MORE_DATA)
			{
				buf.Capacity = (int) ++bufSize;
				ret = NativeMethods.MsiGetProperty(hInstall, Name, buf, ref bufSize);
			}
			if(ret != 0)
			{
				throw new InstallerException(ret); 
			}
			return buf.ToString();
		}

		/// <summary>
		/// Setzt ein Property
		/// </summary>
		/// <param name="Session">Session-Handle</param>
		/// <param name="Name">Name</param>
		/// <param name="Value">Wert</param>
		internal static void SetProperty(uint Session, string Name, string Value)
		{
			IntPtr hInstall = new IntPtr(Session);

			uint ret = NativeMethods.MsiSetProperty(hInstall, Name, Value);
			if(ret != 0)
			{
				throw new InstallerException(ret); 
			}
		}

		/// <summary>
		/// Eigenschaften des installierten Produktes ermitteln
		/// </summary>
		/// <param name="productCode">Produktcode</param>
		/// <param name="productProperty">Eigenschaft</param>
		/// <returns>Ermittelter Wert</returns>
		internal static string GetProductInfo(string productCode, string productProperty)
		{
			StringBuilder buf = new StringBuilder(80);
			uint bufSize = (uint) buf.Capacity;
			uint ret = NativeMethods.MsiGetProductInfo(productCode, productProperty.ToString(), buf, ref bufSize);
			if(ret != 0)
			{
				if(ret == (uint) NativeMethods.Error.MORE_DATA)
				{
					bufSize++;
					buf = new StringBuilder((int) bufSize);
					ret = NativeMethods.MsiGetProductInfo(productCode, productProperty.ToString(), buf, ref bufSize);
				}
				if(ret != 0)
				{
					if(ret == (uint) NativeMethods.Error.UNKNOWN_PRODUCT)
					{
						throw new ArgumentException("Unknown product: " + productCode, "productCode");
					}
					else
					{
						throw new InstallerException(ret);
					}
				}
			}
			return buf.ToString();

		}
	}
	#endregion

	#region Exceptions

	/// <summary>
	/// Klasse zur Behandlung von nativen Exceptions
	/// </summary>
	[Serializable]
	public class InstallerException : SystemException
	{
		/// <summary>
		/// Message formatieren
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		internal static extern uint FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);

		/// <summary>
		/// Instanz der Exception erstellen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		public InstallerException(uint errorCode):base(GetSystemMessage(errorCode)){}

		/// <summary>
		/// Systemfehlermeldung erzeugen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		/// <returns>Meldung</returns>
		private static string GetSystemMessage(uint errorCode)
		{
			const uint FORMAT_MESSAGE_IGNORE_INSERTS  = 0x00000200;
			const uint FORMAT_MESSAGE_FROM_SYSTEM     = 0x00001000;
			StringBuilder buf = new StringBuilder(1024);
			if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				IntPtr.Zero, (uint) errorCode, 0, buf, (uint) buf.Capacity, IntPtr.Zero) != 0)
			{
				return buf.ToString().Trim();
			}
			else
			{
				return null;
			}
		}
	}

	#endregion

	#region Native
	internal sealed class NativeMethods
	{
		/// <summary>
		/// Property auslesen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiGetProperty(IntPtr hInstall, string szName, StringBuilder szValueBuf, ref uint cchValueBuf);
		
		/// <summary>
		/// Property setzen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiSetProperty(IntPtr hInstall, string szName, string szValue);

		/// <summary>
		/// Produkteigenschaft auslesen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiGetProductInfo(string szProduct, string szProperty, StringBuilder lpValueBuf, ref uint pcchValueBuf);

		/// <summary>
		/// Error Auflistung
		/// </summary>
		internal enum Error : uint
		{
			SUCCESS                =    0,
			MORE_DATA              =  234,
			UNKNOWN_PRODUCT        = 1605

		}
	}
	#endregion
}
