/**************************************************************************
*
* Filename:     Exporter.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Enthlt Funktionen um die mit einen Attribut versehenen Methoden zu exportieren
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Microsoft.WindowsInstaller.NativeExporter
{
	using System;
	using System.Text; 
	using System.IO;
	using System.Globalization;
	using System.Diagnostics;
	using System.Collections;
	using System.ComponentModel;
	using System.Text.RegularExpressions;

	#region Delegates
	/// <summary>
	/// Delegate zur Definition des Events, dass ausgelst wird, wenn eine Aktion durchgefhrt wird
	/// </summary>
	public delegate void ExporterMessageHandler(string Message);
	#endregion

	/// <summary>
	/// Enthlt Funktionen um die mit einen Attribut versehenen Methoden zu exportieren
	/// </summary>
	public class Exporter
	{
		#region Instanzvariablen
		
		/// <summary>
		/// Eingabeassembly
		/// </summary>
		private string inputfile;

		/// <summary>
		/// Ausgabeassembly
		/// </summary>
		private string outputfile;

		/// <summary>
		/// Pfad zu ildasm.exe
		/// </summary>
		private string ildasmpath;

		/// <summary>
		/// Pfad zu ilasm.exe
		/// </summary>
		private string ilasmpath;
		/// <summary>
		/// Encoding
		/// </summary>
		private System.Text.Encoding exporttype;

		/// <summary>
		/// Liste fr dei Win32 Resourcen
		/// </summary>
		private ArrayList resourceList = null;

		#endregion

		#region Events
		/// <summary>
		/// Event dass ausgelst wird, wenn eine Aktion durchgefhrt wird
		/// </summary>
		public event ExporterMessageHandler ExporterMessage;

		#endregion

		#region Initialisierung
		/// <summary>
		/// Erstellt eine Instanz des Objektes zum Exportieren der Funktionen
		/// </summary>
		/// <param name="inputFile">Name und Pfad des Assemblies, das manipuliert werden soll</param>
		/// <param name="outputFile">Name und Pfad des Assemblies, das erstellt werden soll</param>
		/// <param name="exportType">Encodingformat fr die Dateien</param>
		public Exporter(string inputFile, string outputFile, System.Text.Encoding exportType, string ildasmPath, string ilasmPath)
		{
			// Werte bertragen
			this.inputfile = inputFile;
			this.outputfile = outputFile;
			this.exporttype = exportType;
			this.ildasmpath = ildasmPath;
			this.ilasmpath = ilasmPath;
		}

		#endregion

		#region Funktionen exportieren
		/// <summary>
		/// Manipuliert das Assembly, in dem die markierten Funktionen exportiert werden
		/// </summary>
		/// <param name="debug">True, falls Debuginformationen generiert werden sollen</param>
		public void Export(bool Debug)
		{

			// Event feuern
			this.OnExporterMessage("Begin with exporting functions..."); 

			// Temporres Verzeichnis bestimmen
			string tempPath = this.CreateTempSubDirectory(); 

			try
			{
				// IL-Dateien konstruieren
				string ilInFile = Path.Combine(tempPath, "ILInput.il");
				string ilOutFile = Path.Combine(tempPath, "ILOutput.il");

				// Ausgabeordner konstruieren
				string outputPath = Path.GetDirectoryName(this.outputfile);
				if (!Directory.Exists(outputPath) )
					Directory.CreateDirectory(outputPath);

				// Assembly disassemblieren
				this.Disassemble(this.inputfile, ilInFile);  

				// Textreader erstellen
				using (TextReader input = new StreamReader(ilInFile, exporttype))
				{
					// Textwriter erstellen
					using (TextWriter output = new StreamWriter(ilOutFile, false, exporttype))
					{
						// Funktionen exportieren
						this.DllExport(input, output);
					}
				}

				// IL assemblieren
				this.Assemble( ilOutFile, this.outputfile, Debug); 


			}
			catch (Exception ex)
			{
				// Exception
				throw new ApplicationException(ex.Message,ex);  
			}
			finally
			{
				// Temporrs Verzeichnis lschen
				this.DeleteTempSubDirectory( tempPath);
			}

			// Event feuern
			this.OnExporterMessage("Exporting functions completed successfully..."); 
		}

		/// <summary>
		/// Disassembliert das entsprechende Assembly mit ildasm.exe
		/// </summary>
		/// <param name="assemblyFile">Assembly, das in IL umgewandelt werden soll</param>
		/// <param name="ilFile">IL-Datei, die erstellt werden soll</param>
		private void Disassemble(string assemblyFile, string ilFile)
		{
			// Befehlszeile konstruieren
			string args = String.Format(CultureInfo.InvariantCulture, "\"{0}\" /out=\"{1}\" /nobar /linenum {2}",
				Path.Combine(Environment.CurrentDirectory, assemblyFile), Path.GetFileName(ilFile),
				(exporttype == Encoding.Unicode ? " /unicode" : exporttype == Encoding.UTF8 ? " /utf8" : ""));

			// Neue ArrayList erstellen. In die ArrayList werden Win32ResourceFiles aufgegommen,
			// die evtl. im Assembly integriert sind, damit diese beim Assemblieren integriert
			// werden knnen
			resourceList = new ArrayList();

			// Event feuern
			this.OnExporterMessage("Using ildasm.exe with this arguments: {0}", args); 

			// Neuen Prozess erstellen (ildasm.exe)
			ProcessStartInfo psi = new ProcessStartInfo(ildasmpath);
			psi.Arguments = args;
			psi.UseShellExecute = false;
			psi.CreateNoWindow = true;
			psi.WindowStyle = ProcessWindowStyle.Hidden;
			psi.WorkingDirectory = Path.GetDirectoryName(ilFile);
			psi.RedirectStandardError = true;
			Process proc;
			
			try
			{
				// Prozess starten
				proc = Process.Start(psi);
			}
			catch(Win32Exception ex)
			{
				// Message konstruieren
				string message = string.Format("The application cannot find \"{0}\". Make sure, that \"ildasm.exe\" is in the current directory, in the CLR runtime directory or the parent directory is referenced by the environment variable PATH.", ildasmpath);
				throw new FileNotFoundException(message, ildasmpath, ex); 
			}

			// Definition fr die einzulesende Zeile des Outputs. Hierber wird bestimmt,
			// ob Win32 Resourcefiles im Assembly enthalten sind
			string line;
			
			// Zeilen einlesen
			while((line = proc.StandardError.ReadLine()) != null)
			{
				const string resKey = "// WARNING: Created Win32 resource file ";
				if(line.StartsWith(resKey))
				{
					// Name der Datei bestimmen
					string resFile = Path.Combine(Path.GetDirectoryName(ilFile), line.Substring(resKey.Length));

					// Der Arraylist hinzufgen
					resourceList.Add( resFile);
				}
				else
				{
					// Ansonsten -> Zeile ausgeben
					this.OnExporterMessage(line); 
				}
			}

			// Warten bis der Prozess beendet wurde
			proc.WaitForExit();

			if (proc.ExitCode != 0)
			{
				// Wenn ExitCode != 0 -> Exception feuern
				throw new ApplicationException(string.Format("The application cannot disassemble the file \"{0}\". The Process exit code is \"{1}\".", assemblyFile, proc.ExitCode.ToString() ));
			}
		}

		
		/// <summary>
		/// Assembliert das entsprechende IL-File mit ilasm.exe
		/// </summary>
		/// <param name="ilFile">Il-Datei die verwendet werden soll</param>
		/// <param name="assemblyFile">Assembly, das erstellt werden soll</param>
		/// <param name="debug">True, falls Debuginformationen generiert werden sollen</param>
		private void Assemble(string ilFile, string assemblyFile, bool debug)
		{
			// Befehlszeile konstruieren
			string args = String.Format(CultureInfo.InvariantCulture, "\"{0}\" /output=\"{1}\" /nologo /quiet {2}{3}",
				Path.GetFileName(ilFile), Path.Combine(Environment.CurrentDirectory, assemblyFile),
				(Path.GetExtension(assemblyFile).ToLower() == ".dll" ? "/dll" : "/exe"),
				(debug ? " /debug" : ""));

			// Falls Win32 Resourcen vorhanden sind, diese in die Befehlszeile
			// integrieren
			if(resourceList != null)
			{
				// Iteration durch alle Resourcen
				foreach(string res in resourceList)
				{
					// Argumente erweitern
					args = String.Format("{0} /resource=\"{1}\"", args, Path.GetFileName(res));
				}
			}

			// Event feuern
			this.OnExporterMessage("Using ilasm.exe with this arguments: {0}", args); 

			// Neuen Prozess erstellen (ilasm.exe)
			ProcessStartInfo psi = new ProcessStartInfo(ilasmpath);
			psi.Arguments = args;
			psi.UseShellExecute = false;
			psi.CreateNoWindow = true;
			psi.WindowStyle = ProcessWindowStyle.Hidden;
			psi.WorkingDirectory = Path.GetDirectoryName(ilFile);
			psi.RedirectStandardOutput = true;
			Process proc;

			try
			{
				// Prozess starten
				proc = Process.Start(psi);
			}
			catch(Win32Exception ex)
			{
				// Message konstruieren
				string message = string.Format("The application cannot find \"{0}\". Make sure, that \"ilasm.exe\" is in the current directory, in the CLR runtime directory or the parent directory is referenced by the environment variable PATH.", ilasmpath);
				throw new FileNotFoundException(message, ilasmpath, ex); 
			}

			// Output ausgeben
			string line;
			while((line = proc.StandardOutput.ReadLine()) != null)
			{
				if(line.Length > 0 && !line.StartsWith("Assembling '") &&
					!line.StartsWith("Source file is") && !line.StartsWith("Operation completed successfully"))
				{
					// Zeile ausgeben
					this.OnExporterMessage(line); 
				}
			}
			// Warten bis der Prozess beendet wurde
			proc.WaitForExit();

			if (proc.ExitCode != 0)
			{
				// Wenn ExitCode != 0 -> Exception feuern
				throw new ApplicationException(string.Format("The application cannot assemble the file \"{0}\". The Process exit code is \"{1}\".", ilFile, proc.ExitCode.ToString() ));
			}
		}

		/// <summary>
		/// Parsen des IL-Files und Modifizieren des IL-Codes, um die
		/// Funktionen, die mit dem Attribut DllExportAttribute versehen sind,
		/// zu exportieren
		/// </summary>
		/// <param name="input">Textreader fr den Eingabestream</param>
			/// <param name="output">Textreader fr den Ausgabestream</param>
		private void DllExport(TextReader input, TextWriter output)
		{
			// Deklaration der Variablen
			bool corFlagsSet = false;
			bool inDllExportAttributeImpl = false;
			string currentPublicStaticMethod = null;
			IList appendLines = new ArrayList();

			int exportNumber = 0;
			Match match;

			// Event feuern
			this.OnExporterMessage("Exporting functions in progress..."); 

			// Iteration
			string line;
			while((line = input.ReadLine()) != null)
			{
				// Falls die Klasse mit den zu exportierenden Funktionen in einem Namespace 
				// eingebettet wurde, wird die Methodensignatur mehrzeilig in IL gespeichert. 
				// Dieses wird geprft und die nchste Zeile eingelesen
				match = Regex.Match(line, @"\.method\b.*\bpublic\b.*\bstatic\b.*\b(\w+)");
				if(match.Success)
				{
					line += input.ReadLine();
				}

				// Debugausgabe
				Debug.WriteLine(line); 

				// Den Beginn der Attributklasse bestimmen
				// Die Zeichenfolge: ".class ... DllExportAttribute" muss enthalten sein
				match = Regex.Match(line, @"\.class\b.*\bDllExportAttribute");
				if(match.Success)
				{
					inDllExportAttributeImpl = true;
					continue;
				}
			
				// Das Ende der Attributklasse bestimmen
				// Die Zeichenfolge: "// end of class DllExportAttribute" muss enthalten sein
				match = Regex.Match(line, @"\/\/ end of class DllExportAttribute");
				if(match.Success)
				{
					inDllExportAttributeImpl = false;
					continue;
				}

				// Auf enthaltene Methode prfen
				match = Regex.Match(line, @"\.method\b.*\b\w+\(");
				if(match.Success)
				{
					match = Regex.Match(line, @"\.method\b.*\bpublic\b.*\bstatic\b.*\b(\w+)\(");
					if(match.Success)
					{
						// ffentliche, statische Methode vorhanden
						currentPublicStaticMethod = match.Groups[1].Value;
					}
					else
					{
						currentPublicStaticMethod = null;
					}
				}

				// Auf Instanz prfen
				match = Regex.Match(line, @"\.custom instance void\b.*\bDllExportAttribute");
				if(match.Success)
				{
					// Prfen, ob eine ffentliche, statische Methode vorhanden ist
					if(currentPublicStaticMethod != null)
					{
						// Metadaten anfgen
						exportNumber++;
						appendLines.Add(String.Format(".vtfixup [1] int32 fromunmanaged at VT_{0:d2}", exportNumber));
						appendLines.Add(String.Format(".data VT_{0:d2} = int32(0)", exportNumber));

						// In Ausgabe bertragen
						output.WriteLine("    .vtentry {0}:1", exportNumber);
						output.WriteLine("    .export [{0}] as {1}", exportNumber, currentPublicStaticMethod);
						currentPublicStaticMethod = null;
					}
					continue;
				}

				// Weitere Flags setzen
				if(!corFlagsSet)
				{
					match = Regex.Match(line, @"\.corflags\b");
					if(match.Success)
					{
						line = ".corflags 0x00000002";
						corFlagsSet = true;
					}

					match = Regex.Match(line, @"\.class\b");
					if(match.Success)
					{
						output.WriteLine(".corflags 0x00000002");
						corFlagsSet = true;
					}
				}

				// Falls keine Attributklasse vorhanden sit
				if(!inDllExportAttributeImpl)
				{
					output.WriteLine(line);
				}
			}

			// Hinzugefgte Zeilen schreiben
			foreach(string appendLine in appendLines)
			{
				output.WriteLine(appendLine);
			}
		}

		#endregion

		#region Temporre Dateien
		/// <summary>
		/// Erstellt ein temporres Verzeichnis
		/// </summary>
		/// <returns>Gibt den Verzeichnisnamen zurck</returns>
		private string CreateTempSubDirectory()
		{	
			// Temporres Verzeichnis bestimmen
			string tempDir = Path.GetTempPath();
			
			// Zufallszahl generieren und eindeutigen Verzeichnisnamen
			// konstruieren
			string tempSubDir;
			Random rand = new Random();
			do
			{
				// Namen definieren
				tempSubDir = Path.Combine(tempDir, String.Format("EXP{0:X}", rand.Next()));
			}
			while(Directory.Exists(tempSubDir));

			// Directory erstellen
			Directory.CreateDirectory(tempSubDir); 

			// Event feuern
			this.OnExporterMessage("Create temporary Folder: \"{0}\"",tempSubDir); 

			// Verzeichnisnamen zurckgeben
			return tempSubDir;
		}


		/// <summary>
		/// Erstelltes temporres Directory lschen
		/// </summary>
		/// <param name="directoryName">Name des Directories</param>
		private void DeleteTempSubDirectory(string directoryName)
		{
			try
			{	// Falls Directory existiert -> lschen
				if (Directory.Exists(directoryName))
					Directory.Delete(directoryName,true); 

				// Event feuern
				this.OnExporterMessage("Delete temporary Folder: \"{0}\"",directoryName); 
			}
			catch (Exception ex)
			{
				// Exception
				Debug.WriteLine(ex.Message, "DeleteTempSubDirectory"); 
			}
		}
		#endregion

		#region Private Eventhandler
		/// <summary>
		/// Methode zum Feuern der Fortschrittsmeldungen
		/// </summary>
		/// <param name="format">String, der die Formatierungsregeln enthlt</param>
		/// <param name="arg">Argumente die bergeben werden</param>
		private void OnExporterMessage(string format, params object[] arg ) 
		{            
			if (ExporterMessage != null)
				ExporterMessage(string.Format(format,arg));
		}

		/// <summary>
		/// Methode zum Feuern der Fortschrittsmeldungen
		/// </summary>
		/// <param name="Message">Meldung die gefeuert werden soll</param>
		private void OnExporterMessage(string Message ) 
		{            
			if (ExporterMessage != null)
				ExporterMessage(Message);
		}
		#endregion
	}
}
