/**************************************************************************
*
* Filename:     Global.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Klasse mit den Hauptfunktionen der Anwendung
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Microsoft.WindowsInstaller.NativeExporter
{
	using System;
	using System.Reflection; 
	using System.Collections; 
	using System.IO;

	/// <summary>
	/// Klasse mit den Hauptfunktionen der Anwendung
	/// </summary>
	public class Global
	{
		#region Statische Variablen

		/// <summary>
		/// Name der Anwendung
		/// </summary>
		private static readonly string applicationName = GetAssemblyTitle();

		#endregion

		#region Main
		/// <summary>
		/// Haupteinstiegspunkt der Anwendung
		/// </summary>
		/// <param name="args">Argumente die bergeben werden</param>
		/// <returns>Wert der anzeigt, ob die Funktionen erfolgreich beendet wurde</returns>
		[STAThread]
		static int Main(string[] args) 
		{
			try
			{
				// Variablen mit Standardwerten belegen
				string sourceFile = string.Empty;
				string destinationFile = string.Empty;
				string databasefile = string.Empty;
				string streamname = string.Empty; 
				bool debugMode = false;
				System.Text.Encoding encoding = System.Text.Encoding.Default; 

				// Argumente prfen
				if (args.Length < 2 || args[0] == "/?")
				{
					// Ungltige Argumente oder Hilfe
					Console.WriteLine( Global.ShowHelp() );
					return (1);
				}

				// Iteration durch die Argumente
				for (int i=args.GetLowerBound(0); i<= args.GetUpperBound(0); i++)
				{
					if (i==args.GetLowerBound(0))
					{
						// SourceFile
						sourceFile = args[i];
					}
					else if (i==(args.GetLowerBound(0)+1))
					{
						// DestinationFile
						destinationFile = args[i];
					}
					else
					{
						// Prfen ob Debug, Unicode oder Utf8
						string Value = args[i].ToUpper(); 

						// Prfen ob Datenbank angegeben
						if (Value.StartsWith("/DATABASE:"))
							databasefile = GetStringPart("/DATABASE:", args[i]);

						// Prfen ob Streamname
						if (Value.StartsWith("/STREAMNAME:"))
							streamname = GetStringPart("/STREAMNAME:", args[i]);
						
						if (Value == "/DEBUG")
							debugMode = true;
					
						if (Value == "/UNICODE")
							encoding = System.Text.Encoding.Unicode; 

						else if (Value == "/UTF8")
							encoding = System.Text.Encoding.UTF8; 

					}
				}

				// Daten modifizieren
				Global.ModifyData(sourceFile, destinationFile, debugMode, encoding); 

				// Zur Datenbank hinzufgen
				if (databasefile != string.Empty && File.Exists(databasefile))
				{
					if (streamname == string.Empty)
						streamname = "NexpoBinary";

					MsiFunctions.AddBinaryStream(databasefile,destinationFile,streamname);  

					Console.WriteLine("Importing stream completed successfully..."); 
				}

				// Alles erfolgreich
				return (0);
			}
			catch (Exception ex)
			{
				// Fehler aufgetreten
				Console.WriteLine (ex.Message); 
				return (1);
			}
		}
		#endregion

		#region Private Funktionen
		/// <summary>
		/// Ermittelt den relevanten Teil einer Zeichenfolge
		/// </summary>
		/// <param name="argument">Zu extrahierende Zeichen</param>
		/// <param name="fullvalue">Vollstndige Zeichenfolge</param>
		/// <returns>Modifizierte Zeichenfolge</returns>
		private static string GetStringPart(string argument, string fullvalue)
		{
			return(System.Text.RegularExpressions.Regex.Replace(fullvalue, argument, string.Empty, System.Text.RegularExpressions.RegexOptions.IgnoreCase ));
		}

		/// <summary>
		/// Funktion zum tatschlichen Modifizieren des Assemblies
		/// </summary>
		/// <param name="sourceFile">Quelldatei</param>
		/// <param name="destinationFile">Zieldatei</param>
		/// <param name="debugMode">True, falls Debuginformationen generiert werden sollen</param>
		/// <param name="encodingMode">Modus zum Encoding</param>
		private static void ModifyData(string sourceFile, string destinationFile, bool debugMode, System.Text.Encoding encodingMode)
		{
			// Prfen ob Datei existiert
			if (!File.Exists(sourceFile)) 
				throw new FileNotFoundException(string.Format("The application experienced an error trying to open the file \"{0}\"",sourceFile),sourceFile);  
		
			// Pfade fr ildasm.exe und ilasm.exe bestimmen
			string ildasmFile = Global.GetFullFilePath("ildasm.exe");
			string ilasmFile = Global.GetFullFilePath("ilasm.exe");
		
			// Instanz erstellen und Methode ausfhren
			Exporter assemblyExport = new Exporter(sourceFile, destinationFile, encodingMode, ildasmFile, ilasmFile );
			assemblyExport.ExporterMessage +=new ExporterMessageHandler(assemblyExport_ExporterMessage);
			assemblyExport.Export( debugMode);
		}


		/// <summary>
		/// Konstruiert den Banner zur Darstellung in der Konsole
		/// </summary>
		/// <returns>Gibt die Zeichenfolge zurck</returns>
		private static string ShowHelp()
		{
			// StringBuilder erzeugen
			System.Text.StringBuilder sb = new System.Text.StringBuilder();

			sb.AppendFormat("{0} version {1}\n\n", applicationName, System.Reflection.Assembly.GetExecutingAssembly().GetName().Version);  
			sb.Append("NativeXporter will modify the current assembly to export the specified methods.\n\n");
			sb.Append("Syntax: NativeXporter <sourcefile> <outputfile> [/database:<FileName>] [/streamname:<Name>] [/debug] [/unicode or /utf8]\n\n"); 
			sb.Append("sourcefile:\t\t Specify the source Assembly.\n"); 
			sb.Append("outputfile:\t\t Compile to file with specified name.\n"); 
			sb.Append("/database:<FileName>\t Import File in installer database.\n"); 
			sb.Append("/streamname:<Name>\t Key of the row in the binary table.\n"); 
			sb.Append("/debug:\t\t\t Include debug information.\n"); 
			sb.Append("/utf8:\t\t\t Use UTF-8 encoding for output (default - ANSI).\n"); 
			sb.Append("/unicode:\t\t Use UNICODE encoding for output.\n\n"); 
			sb.Append("Example: NativeXporter infile.dll outfile.dll /database:rtm.msi /debug /unicode\n"); 

			return (sb.ToString()); 
		}

		/// <summary>
		/// Ermittelt den Pfad zu der Datei
		/// </summary>
		/// <param name="fileName">Name der Datei</param>
		/// <returns>Gibt den vollstndigen Pfad zurck, falls sich die Datei im aktuellen
		/// Verzeichnis oder im Verzeichnis der Runtime befindet. Wird die Datei nicht
		/// gefunden muss das Verzeichnis im PATH stehen.</returns>
		private static string GetFullFilePath(string fileName)
		{
			// Dateinamen bergeben
			string path = fileName;
			try
			{
				// Prfen ob die Datei im aktuellen Verzeichnis ist
				string path2 = Path.Combine(Environment.CurrentDirectory, fileName);
				if(File.Exists(path2))
				{
					path = path2;
				}
				else
				{
					// Prfen, ob die Datei im Verzeichnis der Runtime ist
					string path3 = Path.Combine(System.Runtime.InteropServices.RuntimeEnvironment.GetRuntimeDirectory(),fileName);
					if(File.Exists(path3))
					{
						path = path3;
					}
				}
			}
			catch(Exception) { }
			
			
			// Rckgabe des Pfades
			return path;
		}

		/// <summary>
		/// Funktion zum Darstellen der Fortschrittsmedungen
		/// </summary>
		/// <param name="Message">Meldung die angezeigt werden soll</param>
		private static void assemblyExport_ExporterMessage(string Message)
		{
			// Ausgabe
			Console.WriteLine( Message );
		}

		/// <summary>
		/// Ermittelt den Titel des Assemblies
		/// </summary>
		/// <returns>Gibt den Titel zurck</returns>
		private static string GetAssemblyTitle()
		{
			// Variable deklarieren
			string title = string.Empty; 

			try
			{
				// Assembly bestimmen
				System.Reflection.Assembly Asm = System.Reflection.Assembly.GetExecutingAssembly();

				// AssemblyInformationen auslesen
				AssemblyTitleAttribute assemblytitle = (AssemblyTitleAttribute)AssemblyTitleAttribute.GetCustomAttribute(Asm,typeof(AssemblyTitleAttribute));

				title = assemblytitle.Title; 
			}
			catch 
			{
				// Exception
				title = "Microsoft (R) .NET Assembly Native DLL-Exporter";
			}

			return (title);
		}
		#endregion
	}
}
