/**************************************************************************
*
* Filename:     MsiFunctions.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Funktionen zum Integrieren des Assemblies in die MSI-Datenbank
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Microsoft.WindowsInstaller.NativeExporter
{
	using System;
	using System.Text; 
	using System.Runtime.InteropServices; 

	/// <summary>
	/// Binrdatei der Tabelle Binary hinzufgen
	/// </summary>
	public class MsiFunctions
	{

		#region Konstanten
		/// <summary>
		/// Funktion erfolgreich ausgefhrt
		/// </summary>
		private const uint ERROR_SUCCESS = 0;

		#endregion

		#region Native Funktionen
		/// <summary>
		/// Datenbank ffnen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		private static extern uint MsiOpenDatabase(string szDatabasePath, IntPtr uiOpenMode, out IntPtr hDatabase);

		/// <summary>
		/// View ffnen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		private static extern uint MsiDatabaseOpenView(IntPtr hDatabase, string szQuery, out IntPtr hView);

		/// <summary>
		/// Handle schlieen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		private static extern uint MsiCloseHandle(IntPtr hAny);

		/// <summary>
		/// Record erstellen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		private static extern IntPtr MsiCreateRecord(uint cParams);

		/// <summary>
		/// Stream setzen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		private static extern uint MsiRecordSetStream(IntPtr hRecord, uint iField, string szFilePath);

		/// <summary>
		/// String setzen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		private static extern uint MsiRecordSetString(IntPtr hRecord, uint iField, string szValue);

		/// <summary>
		/// View modifizieren
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		private static extern uint MsiViewModify(IntPtr hView, int iModifyMode, IntPtr hRecord);

		/// <summary>
		/// Commit
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		private static extern uint MsiDatabaseCommit(IntPtr hDatabase);

		#endregion

		#region Methoden
		/// <summary>
		/// Binary-Stream hinzufgen
		/// </summary>
		/// <param name="databaseFileName">Referenz zur Installer-Datenbank</param>
		/// <param name="streamFileName">Referenz auf die Binrdatei</param>
		/// <param name="streamName">Name des Streams</param>
		public static void AddBinaryStream(string databaseFileName, string streamFileName, string streamName)
		{
			// Modus zum ffnen definierem
			IntPtr mode = new IntPtr(1); // Mode zum ffnen der Datenbank (Transact)
			
			// Handle fr geffnete Datenbank definieren
			IntPtr hDb;

			// Datenbank ffnen
			uint ret = MsiOpenDatabase(databaseFileName, mode, out hDb);
			if (ret == ERROR_SUCCESS)
			{
				// Stream hinzufgen
				AssignStream(hDb, streamFileName, streamName);

				// Transaktion abschlieen
				ret = MsiDatabaseCommit(hDb);
				if (ret != ERROR_SUCCESS)
					throw new InstallerException(ret); 

				// Handle der Datenbank schlieen
				MsiCloseHandle(hDb);
			}
			else
			{
				// Exception
				throw new InstallerException(ret);
			}
		}

		/// <summary>
		/// Funktion zum Hinzufgen des Streams
		/// </summary>
		/// <param name="hDb">Handle auf die Datenbank</param>
		/// <param name="streamFileName">Referenz auf die Binrdatei</param>
		/// <param name="streamName">Name des Streams</param>
		private static void AssignStream(IntPtr hDb, string streamFileName, string streamName)
		{
			uint ret;
			// Handle fr View definieren
			IntPtr hView;

			// View ffnen
			if (MsiDatabaseOpenView(hDb, "SELECT `Name`, `Data` FROM `Binary`", out hView) == ERROR_SUCCESS)
			{
				// Handle fr View definieren
				IntPtr hRecord = MsiCreateRecord(2);

				// Name des Streams setzen
				ret = MsiRecordSetString(hRecord, 1, streamName);
				if (ret != ERROR_SUCCESS)
					throw new InstallerException(ret); 

				// Stream setzen
				ret = MsiRecordSetStream(hRecord, 2, streamFileName);
				if (ret != ERROR_SUCCESS)
					throw new InstallerException(ret); 

				// View modifizieren
				ret = MsiViewModify(hView,3,hRecord); // Assign
				if (ret != ERROR_SUCCESS)
					throw new InstallerException(ret); 

				// Handle vom Record schlieen
				MsiCloseHandle(hRecord);

				// Handle vom View schlieen
				MsiCloseHandle(hView);
			}
		}
		#endregion
	}

	#region Exceptions

	/// <summary>
	/// Klasse zur Behandlung von nativen Exceptions
	/// </summary>
	[Serializable]
	public class InstallerException : SystemException
	{
		/// <summary>
		/// Message formatieren
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		internal static extern uint FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);

		/// <summary>
		/// Instanz der Exception erstellen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		public InstallerException(uint errorCode):base(GetSystemMessage(errorCode)){}

		/// <summary>
		/// Systemfehlermeldung erzeugen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		/// <returns>Meldung</returns>
		private static string GetSystemMessage(uint errorCode)
		{
			const uint FORMAT_MESSAGE_IGNORE_INSERTS  = 0x00000200;
			const uint FORMAT_MESSAGE_FROM_SYSTEM     = 0x00001000;
			StringBuilder buf = new StringBuilder(1024);
			if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				IntPtr.Zero, (uint) errorCode, 0, buf, (uint) buf.Capacity, IntPtr.Zero) != 0)
			{
				return buf.ToString().Trim();
			}
			else
			{
				return null;
			}
		}
	}

	#endregion
}
