/**************************************************************************
*
* Filename:     FormatGuid.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Funktionen zur Formatierung einer GUID
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Darwin
{
	using System;
	/// <summary>
	/// Funktionen zur Formatierung einer GUID
	/// </summary>
	public class FormatGuid
	{
		#region Enumeration
		/// <summary>
		/// Enumeration der Formatierungstypen
		/// </summary>
		public enum GuidFormatStyle
		{
			/// <summary>
			/// Vollstndige Guid
			/// </summary>
			Full,
			/// <summary>
			/// Formatierung zur Bezeichnung von Identifiers, die aus
			/// Mergemodulen verwendet werden
			/// </summary>
			Merge,

			/// <summary>
			/// Komprimierte Darstellung, wie Sie in der Registry verwendet wird
			/// </summary>
			Compressed
		}
		#endregion

		#region Konstanten
		/// <summary>
		/// Lnge der GUID
		/// </summary>
		private const int lenGUID = 38;
		
		#endregion

		#region Statische Variablen
		/// <summary>
		/// Reihenfolge der Werte
		/// </summary>
		private static int[] OrderGUID = new int[32] {8, 7, 6, 5, 4, 3, 2, 1, 13, 12, 11, 10, 18, 17, 16, 15, 21, 20, 23, 22, 26, 25, 28, 27, 30, 29, 32, 31, 34, 33, 36, 35}; 
		
		/// <summary>
		/// Positionen, an denen "-" stehen 
		/// </summary>
		private static int[] OrderDash = new int[4] {9, 14, 19, 24};

		#endregion

		#region Methoden
		/// <summary>
		/// Stellt eine GUID in verschiedenen Formaten dar
		/// </summary>
		/// <param name="guidValue">Zeichenfolge, die die GUID reprsentiert</param>
		/// <param name="formatStyle">Formtierung die verwendet werden soll</param>
		/// <returns>Gibt die formatierte Zeichenfolge zurck</returns>
		public static string PackGUID(string guidValue, GuidFormatStyle formatStyle)
		{
			// Rckgabewert initialisieren
			string guid = string.Empty; 

			// Guid in Char-Array umwandeln
			char[] chguid = guidValue.ToCharArray(); 

			// berprfen, ob die Guid die richtige Lnge aufweist
			if (guidValue.Length != lenGUID)
				throw new ApplicationException(string.Format("Die GUID \"{0}\" ist ungltig. Die Anzahl der Zeichen betrgt {1}. Eine gltige GUID fr den Windows Installer besteht aus {2} Zeichen.", guidValue, guidValue.Length.ToString(), lenGUID.ToString()));   

			// berprfen ob die GUID mit { beginnt und mit } endet
			if (chguid[0] != '{' || chguid[lenGUID-1] != '}')
				throw new ApplicationException(string.Format("Die GUID \"{0}\" ist ungltig. Eine gltige GUID muss mit dem Zeichen \"{{\" beginnen und mit dem Zeichen \"}}\" enden.", guidValue));  

			try 
			{	// Prfen ob gltige GUID
				System.Guid _guid = new Guid(guidValue); }
			catch 
			{	// Falls nicht -> Exception
				throw new ApplicationException(string.Format("Die GUID \"{0}\" ist ungltig. Eine gltige GUID muss mit dem Zeichen \"{{\" beginnen, mit dem Zeichen \"}}\" enden und das Format \"{{XXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX}}\" aufweisen.", guidValue));  }

			// Prfung des Formatierungsstiles
			if (formatStyle == GuidFormatStyle.Full )
			{
				// Standardmige Darstellung
				guid = guidValue.ToUpper(); 
			}
			else if (formatStyle == GuidFormatStyle.Compressed  )
			{
				// Komprimierte Darstellung: z.B. E23B88E80FDC7DA47B8DF1C6E42C2E15
				// Iteration durch alle Werte der OrderGUID
				foreach (uint Value in OrderGUID)
				{
					// Einzelne Werte an die Zeichenfolge anfgen
					guid += chguid[Value].ToString();
				}
			}
			else if (formatStyle == GuidFormatStyle.Merge   )
			{
				// Darstellung fr Mergemodule: z.B. 8E88B32E_CDF0_4AD7_B7D8_1F6C4EC2E251
				guid = guidValue.Replace("-","_" ); 
				guid = guid.Replace("{",string.Empty); 
				guid = guid.Replace("}",string.Empty ); 
			}

			// Rckgabe des ermittelten Wertes
			return (guid);
		}

		/// <summary>
		/// Ermittelt aus den verschiedenen Formaten eine Guid
		/// </summary>
		/// <param name="squidValue">Zeichenfolge, die umgewandelt werden soll</param>
		/// <param name="formatStyle">Formtierung die verwendet werden soll</param>
		/// <returns>Gibt die formatierte Zeichenfolge zurck</returns>
		public static string UnPackGUID(string squidValue, GuidFormatStyle formatStyle)
		{
			// CharArray mit der Anzahl der Zeichen definieren
			char[] posguid = new char[lenGUID];

			// Guid in Char-Array umwandeln
			char[] chguid = squidValue.ToCharArray(); 

			// Prfung des Formatierungsstiles
			if (formatStyle == GuidFormatStyle.Compressed  )
			{
				// Komprimierte Darstellung: z.B. E23B88E80FDC7DA47B8DF1C6E42C2E15

				// berprfen, ob die Squid die richtige Lnge aufweist
				if (squidValue.Length != 32)
					throw new ApplicationException(string.Format("Die komprimierte GUID \"{0}\" ist ungltig, eine solche Guid muss immer aus \"32\" Zeichen bestehen.", squidValue));  
				
				// Iteration durch alle Werte der OrderGUID
				int i = 0;
				foreach (uint Value in OrderGUID)
				{
					// Einzelne Werte den Array zuordnen
					posguid[Value] = chguid[i++];
				}
			}
			else if (formatStyle == GuidFormatStyle.Merge   )
			{
				// berprfen, ob die Squid die richtige Lnge aufweist
				if (squidValue.Length != 36)
					throw new ApplicationException(string.Format("Die ID fr das Mergemodul \"{0}\" ist ungltig, eine solche ID muss immer aus \"36\" Zeichen bestehen.", squidValue));  

				// Darstellung fr Mergemodule: z.B. 8E88B32E_CDF0_4AD7_B7D8_1F6C4EC2E251
				for (int i=0; i< chguid.Length;i++) 
				{
					posguid[i+1] = chguid[i];
				}
			}

			// Array anhand weiterer Formatierungsregeln anpassen
			foreach(int Value in OrderDash)
			{
				posguid[Value] = '-';
			}
			posguid[0]='{';
			posguid[lenGUID-1]='}';

			// Rckgabe
			return (new string(posguid));
		}


		#endregion
	}
}
