/**************************************************************************
*
* Filename:     MsiFunctions.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Native Windows Installer Funktionen
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace Darwin
{
	using System;
	using System.Text; 
	using System.Runtime.InteropServices; 

	/// <summary>
	/// Summary description for MsiFunctions.
	/// </summary>
	public class MsiFunctions
	{
		#region Enums
		/// <summary>
		/// Installationsstatus
		/// </summary>
		public enum InstallState : int
		{
			NotUsed        = -7,
			BadConfig      = -6,
			Incomplete     = -5,
			SourceAbsent   = -4,
			MoreData       = -3,
			InvalidArgument= -2,
			Unknown        = -1,
			Broken         =  0,
			Advertised     =  1,
			Removed        =  1,
			Absent         =  2,
			Local          =  3,
			Source         =  4,
			Default        =  5,
		}
		#endregion

		#region Konstanten

		/// <summary>
		/// Buffer ist zu klein
		/// </summary>
		internal const uint ERROR_MORE_DATA	=  234;
		#endregion

		#region Funktionen
		/// <summary>
		/// Ermittelt Informationen ber das Produkt, das Fetaure und/oder die Komponente
		/// </summary>
		/// <param name="Descriptor">Der Descriptor fr das Produkt, das Feature, die Komponente oder eine SQUID</param>
		/// <param name="ProductCode">Stringbuilder der den ProductCode bzw. die Guid aufnehmen soll. Die Kapazitt muss mintestens 39 Zeichen betragen.</param>
		/// <param name="FeatureName">Stringbuilder der den FeatureName aufnehmen soll. Die Kapazitt muss mintestens 39 Zeichen betragen.</param>
		/// <param name="ComponentCode">Stringbuilder der den ComponentCode aufnehmen soll. Die Kapazitt muss mintestens 39 Zeichen betragen.</param>
		/// <param name="Offset">Gibt den Offset des Descriptors an, falls noch zustzliche Argumente bergeben wurden</param>
		/// <returns>ERROR_SUCCESS, falls die Funktion fehlerfrei durchgefhrt wurde</returns>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiDecomposeDescriptor(string Descriptor, StringBuilder ProductCode, StringBuilder FeatureName, StringBuilder ComponentCode, ref uint Offset);
	
		/// <summary>
		/// Ermittelt Eigenschaften des Produktes
		/// </summary>
		/// <param name="szProduct">Productcode oder Darwin Descriptor</param>
		/// <param name="szProperty">Name des Properties</param>
		/// <param name="lpValueBuf">Buffer</param>
		/// <param name="pcchValueBuf">Gre des Buffers</param>
		/// <returns>Gibt den ermittelten Wert zurck</returns>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiGetProductInfo(string szProduct, string szProperty, StringBuilder lpValueBuf, ref uint pcchValueBuf);

		/// <summary>
		/// Ermittelt den Installationsstatus des Produktes
		/// </summary>
		/// <param name="szProduct">Productcode oder Darwin Descriptor</param>
		/// <returns>Status</returns>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern int  MsiQueryProductState(string szProduct);

		/// <summary>
		/// Ermittelt den Installationsstatus des Features
		/// </summary>
		/// <param name="szProduct">Productcode oder Darwin Descriptor</param>
		/// <param name="szFeature">Name des Features</param>
		/// <returns>Status</returns>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern int MsiQueryFeatureState(string szProduct, string szFeature);

		/// <summary>
		/// Ermittelt den Status der Komponente oder das Installationsverzeichnis
		/// </summary>
		/// <param name="szProduct">Productcode oder Darwin Descriptor</param>
		/// <param name="szComponent">KomponentenID</param>
		/// <param name="lpPathBuf">Buffer</param>
		/// <param name="pcchBuf">Gre des Buffers</param>
		/// <returns>Ermittelter Wert</returns>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern int  MsiGetComponentPath(string szProduct, string szComponent, StringBuilder lpPathBuf, ref uint pcchBuf);
		#endregion	}

		#region Methoden
		/// <summary>
		/// Ermittelt den Namen des Produktes
		/// </summary>
		/// <param name="ProductCode">Productcode</param>
		/// <returns>Name des Produktes</returns>
		internal static string GetProductName(string ProductCode)
		{
			// Stringbuilder initialisieren
			StringBuilder buf = new StringBuilder(80);
			uint bufSize = (uint) buf.Capacity;
			
			// Produktinfo abrufen
			uint ret = MsiFunctions.MsiGetProductInfo(ProductCode, "InstalledProductName", buf, ref bufSize);
			if(ret != 0)
			{
				if(ret == MsiFunctions.ERROR_MORE_DATA)
				{
					// Falls Buffer zu klein -> Erhhen
					bufSize++;
					buf = new StringBuilder((int) bufSize);
					ret = MsiFunctions.MsiGetProductInfo(ProductCode, "InstalledProductName", buf, ref bufSize);
				}
			}

			// Rckgabe
			return (buf.ToString());
		}

		/// <summary>
		/// Ermittelt den Status des Produktes
		/// </summary>
		/// <param name="ProductCode">Productcode</param>
		/// <returns>Gibt den Status des Produktes zurck</returns>
		internal static string GetProductState(string ProductCode)
		{
			int installState = MsiFunctions.MsiQueryProductState(ProductCode);
			return ((InstallState)installState).ToString();
		}

		/// <summary>
		/// Ermittelt den Status des Features
		/// </summary>
		/// <param name="ProductCode">Productcode</param>
		/// <param name="FeatureName">Name des Features</param>
		/// <returns>Gibt den Status des Features zurck</returns>
		internal static string GetFeatureState(string ProductCode, string FeatureName)
		{

			// Abbruch, falls keine Featurename vorliegt
			if (FeatureName == string.Empty)
				return (string.Empty); 

			int installState = MsiFunctions.MsiQueryFeatureState(ProductCode, FeatureName);
			return ((InstallState)installState).ToString();
		}

		/// <summary>
		/// Ermittelt den Status der Komponente
		/// </summary>
		/// <param name="ProductCode">Productcode</param>
		/// <param name="ComponentCode">Componentcode</param>
		/// <returns>Gibt den Status der Komponente zurck</returns>
		internal static string GetComponentState(string ProductCode, string ComponentCode)
		{
			// Abbruch, falls keine Komponente vorliegt
			if (ComponentCode == string.Empty)
				return (string.Empty); 
	
			uint bufSize = 0;
			int installState = MsiFunctions.MsiGetComponentPath(ProductCode, ComponentCode, null, ref bufSize);
			return ((InstallState)installState).ToString();
		}

		/// <summary>
		/// Ermittelt den Pfad der Komponente
		/// </summary>
		/// <param name="ProductCode">Productcode</param>
		/// <param name="ComponentCode">Componentcode</param>
		/// <returns>Gibt den Pfad der Komponente zurck</returns>
		internal static string GetComponentPath(string ProductCode, string ComponentCode)
		{
			// Abbruch, falls keine Komponente vorliegt
			if (ComponentCode == string.Empty)
				return (string.Empty); 
	
			StringBuilder buf = new StringBuilder(80);
			uint bufSize = (uint) buf.Capacity;
			int installState = MsiFunctions.MsiGetComponentPath(ProductCode, ComponentCode, buf, ref bufSize);
			if(bufSize >= buf.Capacity)
			{
				buf.Capacity = (int) ++bufSize;
				installState = MsiFunctions.MsiGetComponentPath(ProductCode, ComponentCode, buf, ref bufSize);
			}
			return buf.ToString();
		}



		#endregion
	}
}
