/**************************************************************************
*
* Filename:     MsiFunctions.cs
* Author:       Andreas Kerl (andreaskerl@msn.com)
*               http://www.microsoft.com/germany/
*
* Description:  Zugriff auf Windows Installer-Funktionen
*				
*
*
* Copyright 2005, Andreas Kerl
* 
**************************************************************************/

namespace ScriptViewer
{
	using System;
	using System.Text;
	using System.Globalization;
	using System.Runtime.InteropServices;

	#region Delegates
	/// <summary>
	/// Delegate zur bermittlung der gelesenen Daten
	/// </summary>
	public delegate void LoadDataCallback(string readOperation);

	#endregion

	#region Windows Installer Funktionen
	/// <summary>
	/// Klasse mit Windows Installer Funktionen
	/// </summary>
	public class MsiFunctions
	{
		#region Konstanten
		/// <summary>
		/// Funktion erfolgreich ausgefhrt
		/// </summary>
		private const uint ERROR_SUCCESS = 0;

		/// <summary>
		/// Zustzliche Daten sind vorhanden
		/// </summary>
		private const uint ERROR_MORE_DATA =  234;

		#endregion

		#region Native Funktionen
		/// <summary>
		/// Datenbank ffnen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiOpenDatabase(string szDatabasePath, IntPtr uiOpenMode, out IntPtr hDatabase);

		/// <summary>
		/// View ffnen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)] 
		internal static extern uint MsiDatabaseOpenView(IntPtr hDatabase, string szQuery, out IntPtr hView);

		/// <summary>
		/// View ausfhren
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern uint MsiViewExecute(IntPtr hView, IntPtr hRecord);

		/// <summary>
		/// Daten abrufen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern uint MsiViewFetch(IntPtr hView, out IntPtr hRecord);

		/// <summary>
		/// Handle schlieen
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern uint MsiCloseHandle(IntPtr hAny);

		/// <summary>
		/// Record formatieren
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern uint MsiFormatRecord(IntPtr hInstall, IntPtr hRecord, StringBuilder szResultBuf, ref uint cchResultBuf);

		/// <summary>
		/// Produktinformationen aus dem Skript extrahieren
		/// </summary>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern uint MsiGetProductInfoFromScript(string szScriptFile, StringBuilder lpProductBuf39, out ushort plgidLanguage, out uint pdwVersion, StringBuilder lpNameBuf, ref uint cchNameBuf, StringBuilder lpPackageBuf, ref uint cchPackageBuf);

		/// <summary>
		/// Version des Windows Installers abrufen
		/// </summary>
		/// <param name="dvi"></param>
		[DllImport("msi.dll", CharSet=CharSet.Unicode)]
		internal static extern void DllGetVersion(uint[] dvi);


		#endregion

		#region Methoden
		/// <summary>
		/// Liest die Skriptdatei
		/// </summary>
		/// <param name="filePath">Pfad zu der Skriptdatei</param>
		/// <param name="dataCallback">Callbackfunktion</param>
		public static void ReadScript(string filePath, LoadDataCallback dataCallback)
		{
			// Modus zum ffnen definierem
			IntPtr mode = new IntPtr(5); // Mode zum ffnen einer Skriptdatei 
			
			// Handle fr geffnete Datenbank definieren
			IntPtr hDb;

			// Datenbank ffnen
			uint ret = MsiOpenDatabase(filePath, mode, out hDb);
			if (ret == ERROR_SUCCESS)
			{
				// Handle fr View definieren
				IntPtr hView;

				// View ffnen
				if (MsiDatabaseOpenView(hDb, " ", out hView) == ERROR_SUCCESS)
				{
					// Execute ausfhren
					if (MsiViewExecute(hView, IntPtr.Zero) == ERROR_SUCCESS)
					{
						// Handle fr View definieren
						IntPtr hRecord;

						while (MsiViewFetch(hView, out hRecord) == ERROR_SUCCESS)
						{
							// Record formatieren
							string recordData = FormatRecord(hRecord);

							// An Calllback bergeben
							dataCallback(recordData);
						}
					}
					// Handle vom View schlieen
					MsiCloseHandle(hView);
				}

				// Handle der Datenbank schlieen
				MsiCloseHandle(hDb);
			}
			else
			{
				// Exception
				throw new InstallerException(ret);
			}
		}

		/// <summary>
		/// Formatiert den ermittelten Datensatz
		/// </summary>
		/// <param name="hRecord">Handle auf den Datensatz</param>
		/// <returns>Formatierte Zeichenfolge</returns>
		private static string FormatRecord(IntPtr hRecord)
		{
			// Stringbuilder instanziieren
			StringBuilder buf = new StringBuilder("");
			
			// Gre des Buffers bestimmen
			uint bufSize = 0;
			
			// Gre des Buffers ermitteln
			uint ret = MsiFormatRecord(IntPtr.Zero, hRecord, buf, ref bufSize);
			if(ret != ERROR_MORE_DATA)
			{
				// Exception
				throw new InstallerException(ret);
			}

			// Buffer anpassen und StringBuilder erstellen
			bufSize++;
			buf = new StringBuilder((int) bufSize);
			
			// Daten auslesen
			ret = MsiFormatRecord(IntPtr.Zero, hRecord, buf, ref bufSize);
			if(ret != 0)
			{
				// Exception
				throw new InstallerException(ret);
			}

			// Ermittelten Wert zurckgeben
			return buf.ToString();
		}
		
		/// <summary>
		/// Ermittelt die Produktinformationen aus einem Skript
		/// </summary>
		/// <param name="filePath">Pfad zu der Skriptdatei</param>
		/// <returns>Produktinformationen</returns>
		public static string GetProductInfoFromScript(string filePath)
		{
			// Variablen deklarien
			ushort lang;
			uint ver;
			StringBuilder productCodeBuf = new StringBuilder(40);
			StringBuilder productNameBuf = new StringBuilder(100);
			StringBuilder packageNameBuf = new StringBuilder(40);
			uint productCodeBufSize = (uint) productCodeBuf.Capacity;
			uint productNameBufSize = (uint) productNameBuf.Capacity;
			uint packageNameBufSize = (uint) packageNameBuf.Capacity;

			// Gre des Buffers bestimmen
			uint ret = MsiGetProductInfoFromScript(filePath, productCodeBuf, out lang, out ver, productNameBuf, ref productNameBufSize,
				packageNameBuf, ref packageNameBufSize);
			if(ret == (uint) ERROR_MORE_DATA)
			{
				// Buffer vergrern und Daten auswerten
				productCodeBuf.Capacity = (int) ++productCodeBufSize;
				productNameBuf.Capacity = (int) ++productNameBufSize;
				packageNameBuf.Capacity = (int) ++packageNameBufSize;
				ret = MsiGetProductInfoFromScript(filePath, productCodeBuf, out lang, out ver, productNameBuf, ref productNameBufSize,
					packageNameBuf, ref packageNameBufSize);
			}
			if(ret != 0)
			{
				throw new InstallerException(ret);
			}

			// Version
			Version version = new Version((int) (ver >> 24), (int) ((ver & 0x00FFFFFF) >> 16), (int) (ver & 0x0000FFFF));

			// Stringbuilder erstellen um Informationen auszugeben
			StringBuilder productInfo = new StringBuilder();
			productInfo.Append("ProductCode: " + productCodeBuf.ToString() + Environment.NewLine);  
			productInfo.Append("Language: " + lang.ToString(CultureInfo.InvariantCulture) + Environment.NewLine);  
			productInfo.Append("VersionString: " + version.ToString() + Environment.NewLine);  
			productInfo.Append("ProductName: " + productNameBuf.ToString() + Environment.NewLine);  
			productInfo.Append("PackageName: " + packageNameBuf.ToString());  

			// Rckgabe
			return (productInfo.ToString() ); 
		}

		/// <summary>
		/// Version des Windows Installer ermitteln
		/// </summary>
		/// <returns>ermittelte Version</returns>
		public static Version GetVersion()
		{
			// VersionInfostruktur initialisieren
			uint[] dllVersionInfo = new uint[5];
			dllVersionInfo[0] = 20;
			
			// Version ermitteln
			DllGetVersion(dllVersionInfo);
			
			// Rckgabe
			return (new Version((int) dllVersionInfo[1], (int) dllVersionInfo[2], (int) dllVersionInfo[3]));
		}

		#endregion
	}
	#endregion

	#region Exceptions

	/// <summary>
	/// Klasse zur Behandlung von nativen Exceptions
	/// </summary>
	[Serializable]
	public class InstallerException : SystemException
	{
		/// <summary>
		/// Message formatieren
		/// </summary>
		[DllImport("kernel32.dll", SetLastError=true)] 
		internal static extern uint FormatMessage(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, StringBuilder lpBuffer, uint nSize, IntPtr Arguments);

		/// <summary>
		/// Instanz der Exception erstellen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		public InstallerException(uint errorCode):base(GetSystemMessage(errorCode)){}

		/// <summary>
		/// Systemfehlermeldung erzeugen
		/// </summary>
		/// <param name="errorCode">Fehlercode</param>
		/// <returns>Meldung</returns>
		private static string GetSystemMessage(uint errorCode)
		{
			const uint FORMAT_MESSAGE_IGNORE_INSERTS  = 0x00000200;
			const uint FORMAT_MESSAGE_FROM_SYSTEM     = 0x00001000;
			StringBuilder buf = new StringBuilder(1024);
			if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				IntPtr.Zero, (uint) errorCode, 0, buf, (uint) buf.Capacity, IntPtr.Zero) != 0)
			{
				return buf.ToString().Trim();
			}
			else
			{
				return null;
			}
		}
	}

	#endregion
}
